{-----------------------------------------------------------------------------
Picture conversion algorithm - ported from SCRPLUS (C) Edward Cree 2010
from C to Delphi.

------------------------------------------------------------------------------
MAIN.PAS - The main program displaying the GUI and interaction with the user.
------------------------------------------------------------------------------

This file is part of Image2ULAPlus.

Image2ULAPlus is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Image2ULAPlus is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------------
The Initial Developer of the Original ULAPlus conversion algorithm is
Edward Cree (C) Copyright 2010. Portions created by Klaus Jahn are
Copyright (C) 2011. All Rights Reserved.

Contributor(s): Edward Cree.

You may retrieve the latest version of this file at the ZX-Modules home page,
located at http://www.zxmodules.de

-------------------------------------------------------------------------------
Cross-referenced components and libraries that are not part of this source:

- Delphi 2006 components (c) 1995-2004 Borland Software Corporation
-----------------------------------------------------------------------------}

Unit main;

Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ActnList, StdActns, Menus, Buttons, ComCtrls,
  AppEvnts, ImgList, ToolWin,
  ulaplus, renderer;

Type
  TMainForm = Class(TForm)
    TopPanel: TPanel;
    LeftImagePanel: TPanel;
    ImageSplitter: TSplitter;
    RightImagePanel: TPanel;
    SourceImageLabel: TLabel;
    DestImageLabel: TLabel;
    SourceImage: TImage;
    DestImage: TImage;
    ActionList1: TActionList;
    FileOpenAction: TFileOpen;
    BottomPanel: TPanel;
    RenderAction: TAction;
    Splitter1: TSplitter;
    RulerLeftPanel: TPanel;
    YL_Panel: TPanel;
    YL_ScrollBar: TScrollBar;
    YL_Edit: TEdit;
    YL_Label: TLabel;
    YH_Panel: TPanel;
    YH_Label: TLabel;
    YH_ScrollBar: TScrollBar;
    YH_Edit: TEdit;
    UH_Panel: TPanel;
    UH_Label: TLabel;
    UH_ScrollBar: TScrollBar;
    UH_Edit: TEdit;
    UL_Panel: TPanel;
    UL_Label: TLabel;
    UL_ScrollBar: TScrollBar;
    UL_Edit: TEdit;
    VL_Panel: TPanel;
    VL_Label: TLabel;
    VL_ScrollBar: TScrollBar;
    VL_Edit: TEdit;
    VH_Panel: TPanel;
    VH_Label: TLabel;
    VH_ScrollBar: TScrollBar;
    VH_Edit: TEdit;
    ApplicationEvents1: TApplicationEvents;
    FileOpenPaletteAction: TFileOpen;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ImageList1: TImageList;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    RulerRightPanel: TPanel;
    MaxColors_Panel: TPanel;
    MaxColors_Edit: TEdit;
    Bevel1: TBevel;
    Bevel2: TBevel;
    DitheringFlowCheckBox: TCheckBox;
    DitheringCheckBox: TCheckBox;
    Bevel4: TBevel;
    PalettePanel: TPanel;
    Label3: TLabel;
    PaletteImage: TImage;
    CLUT_Panel: TPanel;
    CLUT0_CheckBox: TCheckBox;
    CLUT1_CheckBox: TCheckBox;
    CLUT2_CheckBox: TCheckBox;
    CLUT3_CheckBox: TCheckBox;
    CalculatePaletteCheckBox: TCheckBox;
    Bevel6: TBevel;
    ToolButton4: TToolButton;
    FileSaveAsAction: TFileSaveAs;
    StretchCheckBox: TCheckBox;
    TimexCheckBox: TCheckBox;
    ResetValuesAction: TAction;
    BitBtn1: TBitBtn;
    Bevel3: TBevel;
    Bevel5: TBevel;
    Bevel7: TBevel;
    Bevel8: TBevel;
    Y_Gradient: TImage;
    U_Gradient: TImage;
    V_Gradient: TImage;
    Depress_Panel: TPanel;
    Depress_Label: TLabel;
    Depress_ScrollBar: TScrollBar;
    Depress_Edit: TEdit;
    Bevel9: TBevel;
    Procedure TopPanelResize(Sender: TObject);
    Procedure FileOpenActionAccept(Sender: TObject);
    Procedure FormCreate(Sender: TObject);
    Procedure RenderActionExecute(Sender: TObject);
    Procedure YL_ScrollBarChange(Sender: TObject);
    Procedure YH_ScrollBarChange(Sender: TObject);
    Procedure UL_ScrollBarChange(Sender: TObject);
    Procedure UH_ScrollBarChange(Sender: TObject);
    Procedure VL_ScrollBarChange(Sender: TObject);
    Procedure VH_ScrollBarChange(Sender: TObject);
    Procedure YL_EditChange(Sender: TObject);
    Procedure YH_EditChange(Sender: TObject);
    Procedure UL_EditChange(Sender: TObject);
    Procedure UH_EditChange(Sender: TObject);
    Procedure VL_EditChange(Sender: TObject);
    Procedure VH_EditChange(Sender: TObject);
    Procedure Depress_ScrollBarChange(Sender: TObject);
    Procedure Depress_EditChange(Sender: TObject);
    Procedure FileOpenPaletteActionAccept(Sender: TObject);
    Procedure MaxColors_EditChange(Sender: TObject);
    Procedure CLUT_CheckBoxClick(Sender: TObject);
    Procedure TimexCheckBoxClick(Sender: TObject);
    Procedure StretchCheckBoxClick(Sender: TObject);
    Procedure CalculatePaletteCheckBoxClick(Sender: TObject);
    Procedure ResetValuesActionExecute(Sender: TObject);
    Procedure FormShow(Sender: TObject);
    Procedure FileSaveAsActionAccept(Sender: TObject);
    Procedure DitheringCheckBoxClick(Sender: TObject);
    Procedure DitheringFlowCheckBoxClick(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FileSaveAsActionBeforeExecute(Sender: TObject);
    procedure FileOpenActionBeforeExecute(Sender: TObject);
  Private
    { Private-Deklarationen }
    FUpdatingLevel:Integer;
    FFormatSettings:TFormatSettings;
    FFirst:Boolean;
    Procedure UpdateEdits(Sender:TObject);
    Procedure UpdatePalette(Sender:TObject);
    Procedure UpdateBitmap(Sender:TObject);
  Public
    { Public-Deklarationen }
  End;

var
  MainForm: TMainForm;

Implementation

{$R *.dfm}

Uses
  Math,
  yuv_helper;

{---------------------- Object methods of mainform ------------------------------}

Procedure TMainForm.FormCreate(Sender: TObject);
Begin
  GetLocaleFormatSettings(GetThreadLocale,FFormatSettings);
  FUpdatingLevel:=0;
  ULAPlusRenderer.OnUpdateEdits:=UpdateEdits;
  ULAPlusRenderer.OnUpdatePalette:=UpdatePalette;
  ULAPlusRenderer.OnUpdateBitmap:=UpdateBitmap;
  UpdateEdits(Self);
  UpdatePalette(Self);
  Ffirst:=True;
End;

Procedure TMainForm.FormShow(Sender: TObject);
Begin
  If FFirst Then Begin
    UpdateBitmap(Self);
    UpdatePalette(Self);
    UpdateEdits(Self);
    FFirst:=False;
  End; {if}
End;

Procedure TMainForm.FileOpenActionAccept(Sender: TObject);
Begin
  ULAPlusRenderer.ReadImageFile(FileOpenAction.Dialog.FileName);
End;

procedure TMainForm.FileOpenActionBeforeExecute(Sender: TObject);
begin
  FileOpenAction.Dialog.Filter:=ULAPlusRenderer.GetInputFileFilters;
end;

Procedure TMainForm.FileOpenPaletteActionAccept(Sender: TObject);
Begin
  ULAPlusRenderer.ReadPaletteFile(FileOpenPaletteAction.Dialog.FileName);
End;

Procedure TMainForm.FileSaveAsActionAccept(Sender: TObject);
Begin
  ULAPlusRenderer.WriteImageFile(FileSaveAsAction.Dialog.FileName);
End;

procedure TMainForm.FileSaveAsActionBeforeExecute(Sender: TObject);
begin
  FileSaveAsAction.Dialog.Filter:=ULAPlusRenderer.GetOutputFileFilters;
end;

Procedure TMainForm.MaxColors_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(MaxColors_Edit.Text,Value) And (Value>=8) Then
    ULAPlusRenderer.MaxColours:=Value;
End;

Procedure TMainForm.RenderActionExecute(Sender: TObject);
Begin
  If ULAPlusRenderer.Render(True,True) Then 
    DestImage.Picture.Assign(ULAPlusRenderer.OutputBitmap);
End;

Procedure TMainForm.ResetValuesActionExecute(Sender: TObject);
Begin
  ULAPlusRenderer.ResetValues;
End;

Procedure TMainForm.StretchCheckBoxClick(Sender: TObject);
Begin
  ULAPlusRenderer.Stretch:=StretchCheckbox.Checked;
End;

Procedure TMainForm.TimexCheckBoxClick(Sender: TObject);
Begin
  ULAPlusRenderer.Timex:=TimexCheckBox.Checked;
End;

Procedure TMainForm.TopPanelResize(Sender: TObject);
Begin
  LeftImagePanel.Width:=(TopPanel.ClientWidth-ImageSplitter.width) Div 2;
End;

Procedure TMainForm.UH_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(UH_Edit.Text,Value) Then
    ULAPlusRenderer.UH:=IntToByte(Value);
End;

Procedure TMainForm.UH_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.UH:=UH_ScrollBar.Position;
End;

Procedure TMainForm.UL_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(UL_Edit.Text,Value) Then
    ULAPlusRenderer.UL:=IntToByte(Value);
End;

Procedure TMainForm.UL_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.UL:=UL_ScrollBar.Position;
End;

Procedure TMainForm.VH_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(VH_Edit.Text,Value) Then
    ULAPlusRenderer.VH:=Value;
End;

Procedure TMainForm.VH_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.VH:=VH_ScrollBar.Position;
End;

Procedure TMainForm.VL_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(VL_Edit.Text,Value) Then
    ULAPlusRenderer.VL:=IntToByte(Value);
End;

Procedure TMainForm.VL_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.VL:=VL_ScrollBar.Position;
End;

Procedure TMainForm.YH_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(YH_Edit.Text,Value) Then
    ULAPlusRenderer.YH:=IntToByte(Value);
End;

Procedure TMainForm.YH_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.YH:=YH_ScrollBar.Position;
End;

Procedure TMainForm.YL_EditChange(Sender: TObject);
var
  Value:Integer;
Begin
  If TryStrToInt(YL_Edit.Text,Value) Then
    ULAPlusRenderer.YL:=IntToByte(Value);
End;

Procedure TMainForm.YL_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.YL:=YL_ScrollBar.Position;
End;

Procedure TMainForm.Depress_EditChange(Sender: TObject);
var
  Value:extended;
Begin
  If TryStrToFloat(Depress_Edit.Text,Value,FFormatSettings) Then
    ULAPlusRenderer.Depress:=Value;
End;

Procedure TMainForm.Depress_ScrollBarChange(Sender: TObject);
Begin
  ULAPlusRenderer.Depress:=Depress_ScrollBar.Position/1000;
End;

Procedure TMainForm.DitheringCheckBoxClick(Sender: TObject);
Begin
  ULAPlusRenderer.Dither:=DitheringCheckBox.Checked;
End;

Procedure TMainForm.DitheringFlowCheckBoxClick(Sender: TObject);
Begin
  ULAPlusRenderer.DitherFlow:=DitheringFlowCheckBox.Checked;
End;

Procedure TMainForm.CalculatePaletteCheckBoxClick(Sender: TObject);
Begin
  ULAPlusRenderer.CalculatePalette:=CalculatePaletteCheckBox.Checked;
End;

Procedure TMainForm.CLUT_CheckBoxClick(Sender: TObject);
Begin
  If FUpdatingLevel=0 Then
    ULAPlusRenderer.ClutMask:=Ord(CLUT0_CheckBox.checked)+2*Ord(CLUT1_CheckBox.Checked)+
     4*Ord(CLUT2_CheckBox.Checked)+8*Ord(CLUT3_CheckBox.Checked);
End;

Procedure TMainForm.UpdateEdits;
Begin
  Inc(FUpdatingLevel);
  yL_Scrollbar.Position:=ULAPlusRenderer.YL;
  YH_Scrollbar.Position:=ULAPlusRenderer.YH;
  UL_Scrollbar.Position:=ULAPlusRenderer.UL;
  UH_Scrollbar.Position:=ULAPlusRenderer.UH;
  VL_Scrollbar.Position:=ULAPlusRenderer.VL;
  VH_Scrollbar.Position:=ULAPlusRenderer.VH;
  YL_Edit.Text:=IntToStr(ULAPlusRenderer.YL);
  YH_Edit.Text:=IntToStr(ULAPlusRenderer.YH);
  UL_Edit.Text:=IntToStr(ULAPlusRenderer.UL);
  UH_Edit.Text:=IntToStr(ULAPlusRenderer.UH);
  VL_Edit.Text:=IntToStr(ULAPlusRenderer.VL);
  VH_Edit.Text:=IntToStr(ULAPlusRenderer.VH);
  Depress_ScrollBar.Position:=Round(ULAPlusRenderer.Depress*1000);
  If (Depress_Edit.Text='') Or
    (StrToFloat(Depress_Edit.Text,FFormatSettings)<>ULAPlusRenderer.depress) Then
    Depress_Edit.Text:=FloatToStrF(ULAPlusRenderer.Depress,FFFixed,5,3,FFormatSettings);
  DitheringCheckBox.Checked:=ULAPlusRenderer.Dither;
  DitheringFlowCheckBox.Checked:=ULAPlusRenderer.DitherFlow;
  DitheringFlowCheckBox.Enabled:=DitheringCheckBox.Checked;
  CalculatePaletteCheckBox.Checked:=ULAPlusRenderer.CalculatePalette;
  Clut0_CheckBox.Checked:=ULAPlusRenderer.ClutMask And 1=1;
  Clut1_CheckBox.Checked:=ULAPlusRenderer.ClutMask And 2=2;
  Clut2_CheckBox.Checked:=ULAPlusRenderer.ClutMask And 4=4;
  Clut3_CheckBox.Checked:=ULAPlusRenderer.ClutMask And 8=8;
  MaxColors_Edit.Text:=IntToStr(ULAPlusRenderer.MaxColours);
  TimexCheckBox.Checked:=ULAPlusRenderer.Timex;
  StretchCheckBox.Checked:=ULAPlusRenderer.Stretch;
  FileSaveAsAction.Enabled:=ULAPlusRenderer.OutputType<>ot_none;
  Dec(FUpdatingLevel);
End;

Procedure TMainForm.UpdatePalette(Sender: TObject);
Begin
  ULAPlusRenderer.GetPaletteBitmap(PaletteImage.Picture.Bitmap);
End;

Procedure TMainForm.UpdateBitmap(Sender: TObject);
Begin
  ULAPlusRenderer.GetAdjustedBitmap(SourceImage.Picture.Bitmap);
End;

Function GetPropsName(p:TPictureProperties):String;
Begin
  If p.ULAPlusMode Then Result:='ULA+'
    Else Result:='';
  If p.TimexMode Then Begin
    If Length(Result)>0 Then Result:=Result+', ';
    Result:=Result+'Timex';
  End; {if}
  If Length(Result)>0 Then Result:=' ('+Result+')';
End;

procedure TMainForm.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
begin
  With SourceImage.Picture.Bitmap Do
    If (Width>0) And (Height>0) Then
      SourceImageLabel.Caption:=Format('Source image [%d x %d]%s:',[Width,Height,
      GetPropsName(ULAPlusRenderer.InputProperties)])
      Else SourceImageLabel.Caption:='Source image <empty>:';
  With DestImage.Picture.Bitmap Do
    If (Width>0) And (Height>0) Then
      DestImageLabel.Caption:=Format('Destination image [%d x %d]%s:',[Width,Height,
      GetPropsName(ULAPlusRenderer.OutputProperties)])
      Else DestImageLabel.Caption:='Destination image <empty>:';
end;

End.
