{-----------------------------------------------------------------------------
Picture conversion algorithm - ported from SCRPLUS (C) Edward Cree 2010
from C to Delphi.

------------------------------------------------------------------------------
            PROGRESSES.PAS - Displays a progress bar with STOP button
------------------------------------------------------------------------------

This file is part of Image2ULAPlus.

Image2ULAPlus is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Image2ULAPlus is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------------
The Initial Developer of the Original ULAPlus conversion algorithm is
Edward Cree (C) Copyright 2010. Portions created by Klaus Jahn are
Copyright (C) 2011. All Rights Reserved.

Contributor(s): Edward Cree.

You may retrieve the latest version of this file at the ZX-Modules home page,
located at http://www.zxmodules.de

-------------------------------------------------------------------------------
Cross-referenced components and libraries that are not part of this source:

- Delphi 2006 components (c) 1995-2004 Borland Software Corporation
-----------------------------------------------------------------------------}

Unit progresses;

Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Buttons, ToolWin, ExtCtrls;

Type
  TProgressbarForm = class(TForm)
    ProgressBar1: TProgressBar;
    StopButton: TBitBtn;
    Procedure stopbuttonClick(Sender: TObject);
    Procedure FormClose(Sender: TObject; Var Action: TCloseAction);
  Private
    { Private declarations }
    FManualBreak:Boolean;
    f:Array Of Boolean;
  public
    { Public declarations }
    Property ManualBreak:Boolean Read FManualBreak;
    Procedure Init(max:Integer; Const cap:String; breakable:Boolean);
    Procedure Progress(pos:Integer); Overload;
    Procedure Progress; Overload;
    Procedure SetCaption(Const cap:String);
    Procedure SetMax(max:Integer);
    Procedure Done;
  End;

Var
  ProgressbarForm: TProgressbarForm;

Implementation

{$R *.dfm}

Var
  ApplicationTitle:String;
  MainFormCaption:String;
  MainProgressMax:Integer;

Procedure MainProgress_Init(max:Integer);
Begin
  ApplicationTitle:=Application.Title;
  If Application.MainForm<>ProgressbarForm Then MainFormCaption:=Application.MainForm.Caption;
  MainProgressMax:=max;
  If MainProgressMax=0 Then inc(MainProgressMax);
End;

Procedure MainProgress_Progress(pos:Integer);
Begin
  Application.Title:=ApplicationTitle+' ('+inttostr(pos*100 div MainProgressMax)+' %)';
  If Application.MainForm<>ProgressbarForm Then
    Application.MainForm.Caption:=Application.Title;
End;

Procedure MainProgress_Done;
Begin
  Application.title:=ApplicationTitle;
  If Application.MainForm<>ProgressbarForm Then
    Application.MainForm.Caption:=MainFormCaption;
End;

Procedure TProgressbarForm.Init;
Var
  i:Integer;
Begin
  progressbar1.Min:=0;
  progressbar1.Max:=max;
  progressbar1.Position:=0;
  Caption:=cap;
  MainProgress_Init(max);
  FManualBreak:=False;
  StopButton.Enabled:=breakable;
  Show;
  If breakable Then Begin
    SetLength(f,Screen.FormCount);
    For i:=0 To Screen.FormCount-1 Do Begin
      f[i]:=Screen.Forms[i].Enabled;
      If Screen.Forms[i]<>self Then Screen.Forms[i].Enabled:=False;
    End; {for}
  End; {if}
End;

Procedure TProgressbarForm.setcaption;
Begin
  Caption:=cap;
End;

Procedure TProgressbarForm.setmax;
Begin
  Progressbar1.Max:=max;
  MainProgressMax:=max;
End;

Procedure TProgressbarForm.Progress(pos:Integer);
Begin
  progressbar1.Position:=pos;
  MainProgress_Progress(pos);
  If StopButton.Enabled Then Application.ProcessMessages;
End;

Procedure TProgressbarForm.Progress;
Begin
  If StopButton.enabled Then Application.ProcessMessages;
End;

Procedure TProgressbarForm.Done;
Var
  i:Integer;
Begin
  MainProgress_Done;
  If StopButton.Enabled Then Begin
    For i:=0 To Screen.FormCount-1 Do
      Screen.Forms[i].Enabled:=f[i];
    SetLength(f,0);
  End; {if}
  Hide;
End;

Procedure TProgressbarForm.stopbuttonClick(Sender: TObject);
Begin
  FManualBreak:=True;
End;

Procedure TProgressbarForm.FormClose(Sender: TObject;
  Var Action: TCloseAction);
Begin
  FManualBreak:=True;
End;

End.
