{-----------------------------------------------------------------------------
Picture conversion algorithm - ported from SCRPLUS (C) Edward Cree 2010
from C to Delphi.

------------------------------------------------------------------------------
       ULAPLUS - Library containing tools for ULAPlus palette handling
------------------------------------------------------------------------------

This file is part of Image2ULAPlus.

Image2ULAPlus is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Image2ULAPlus is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------------
The Initial Developer of the Original ULAPlus conversion algorithm is
Edward Cree (C) Copyright 2010. Portions created by Klaus Jahn are
Copyright (C) 2011. All Rights Reserved.

Contributor(s): Edward Cree.

You may retrieve the latest version of this file at the ZX-Modules home page,
located at http://www.zxmodules.de

-------------------------------------------------------------------------------
Cross-referenced components and libraries that are not part of this source:

- Delphi 2006 components (c) 1995-2004 Borland Software Corporation
-----------------------------------------------------------------------------}

Unit ulaplus;

Interface

Uses
  Windows, Graphics;

type
  Tulapluscolors=Array[0..63] Of Byte;

  Tulapluspalette=Array[0..63] Of TColor;

Procedure GetInkPaper(Attribute:Integer; Var InkIndex,PaperIndex:Integer);
Function getRGBfromColor64(Color64:Integer):TColor;
Function getColor64fromRGB(Color:TColor):Integer;
Function colortoRGBQUAD(c:TColor):RGBQUAD;
Procedure SetPalette(Var DestPalette:TULAPlusPalette; SourcePalette:TULAPlusColors); Overload;
Procedure SetPalette(Var DestPalette:TULAPlusPalette; SourcePalette:String); Overload;
Procedure SetPalette(Var DestPalette:TULAPlusColors; SourcePalette:TULAPlusPalette); Overload;
Procedure SetStandardPalette(Var DestPalette:TULAPlusPalette);
Function IsStandardPalette(Var DestPalette:TULAPlusPalette):Boolean;

implementation

Const
  StandardPalette:TULAPlusPalette=
   ($00000000,$00A00000,$000000DC,$00B400E4,$0000D400,$00D4D400,$0000D0D0,$00C8C8C8,
    $00000000,$00A00000,$000000DC,$00B400E4,$0000D400,$00D4D400,$0000D0D0,$00C8C8C8,
    $00000000,$00AC0000,$000000F0,$00DC00FC,$0000F000,$00FCFC00,$0000FCFC,$00FCFCFC,
    $00000000,$00AC0000,$000000F0,$00DC00FC,$0000F000,$00FCFC00,$0000FCFC,$00FCFCFC,
    $00000000,$00A00000,$000000DC,$00B400E4,$0000D400,$00D4D400,$0000D0D0,$00C8C8C8,
    $00000000,$00A00000,$000000DC,$00B400E4,$0000D400,$00D4D400,$0000D0D0,$00C8C8C8,
    $00000000,$00AC0000,$000000F0,$00DC00FC,$0000F000,$00FCFC00,$0000FCFC,$00FCFCFC,
    $00000000,$00AC0000,$000000F0,$00DC00FC,$0000F000,$00FCFC00,$0000FCFC,$00FCFCFC);

Procedure SetPalette(Var DestPalette:TULAPlusPalette; SourcePalette:TULAPlusColors);
Var
  i:Integer;
Begin
  For i := 0 To 63 Do
    DestPalette[i]:=getRGBfromcolor64(SourcePalette[i]);
End;

Procedure SetPalette(Var DestPalette:TULAPlusPalette; SourcePalette:String);
Var
  i:Integer;
Begin
  if length(Sourcepalette)=64 then Begin
    For i := 0 To 63 Do
      DestPalette[i]:=getRGBfromColor64(Ord(SourcePalette[i+1]));
  End; {if}
End;

Procedure SetPalette(Var DestPalette:TULAPlusColors; SourcePalette:TULAPlusPalette); Overload;
Var
  i:Integer;
Begin
  For i := 0 To 63 Do
    DestPalette[i]:=getColor64fromRGB(SourcePalette[i]);
End;

Procedure SetStandardPalette;
Begin
  DestPalette:=StandardPalette;
End;

Procedure GetInkPaper;
Var
  clut:Integer;
Begin
  clut:=(Attribute Shr 2) And $30;
  InkIndex:=  clut Or (Attribute And $07);
  PaperIndex:=clut Or ((Attribute Shr $03) And $07) Or 8;
End;

Function getRGBfromColor64(Color64:Integer):TColor;
Var
  Red,Green,Blue:Integer;
Begin
  Red:=  (Color64 And $1C) Shl 3;
  Green:=(Color64 And $E0);
  Blue:= (Color64 And $03) Shl 6;
  Result:=Red+Green Shl $08+Blue Shl $10;
End;

Function getColor64fromRGB(Color:TColor):Integer;
Var
  Blue,Green,Red:Integer;
Begin
  Blue:= (Color Shr $10) And $FF;
  Green:=(Color Shr $08) And $FF;
  Red:=  Color And $FF;
  Result:=(32*(Green Div 32))+(4*(Red Div 32))+Blue Div 64;
End;

Function colortoRGBQUAD(c:TColor):RGBQUAD;
Begin
  With Result Do Begin
    rgbBlue:= (c Shr $10) And $FF;
    rgbGreen:=(c Shr $08) And $FF;
    rgbRed:=  c And $FF;
    rgbReserved:=0;
  End; {with}
End;

Function IsStandardPalette(Var DestPalette:TULAPlusPalette):Boolean;
Var
  i:Integer;
Begin
  Result:=True;
  For i := 0 To 63 Do
    if DestPalette[i]<>StandardPalette[i] then Begin
      Result:=False;
      Break;
    End; {for}
End;

End.
