{-----------------------------------------------------------------------------
Picture conversion algorithm - ported from SCRPLUS (C) Edward Cree 2010
from C to Delphi.

------------------------------------------------------------------------------
      YUV_HELPER.PAS - Library for YUV to RGB and vice versa conversion
------------------------------------------------------------------------------

This file is part of Image2ULAPlus.

Image2ULAPlus is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Image2ULAPlus is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------------
The Initial Developer of the Original ULAPlus conversion algorithm is
Edward Cree (C) Copyright 2010. Portions created by Klaus Jahn are
Copyright (C) 2011. All Rights Reserved.

Contributor(s): Edward Cree.

You may retrieve the latest version of this file at the ZX-Modules home page,
located at http://www.zxmodules.de

-------------------------------------------------------------------------------
Cross-referenced components and libraries that are not part of this source:

- Delphi 2006 components (c) 1995-2004 Borland Software Corporation
-----------------------------------------------------------------------------}

Unit yuv_helper;

Interface

Uses
  Windows, Graphics;

Type
  TYUVbyte=Packed Record
    IY,IU,IV:Byte;
  End; {record}

  TYUVint=Packed record
    IY,IU,IV:SmallInt;
  End; {record}

  TYUVintn=Packed record
    IY,IU,IV:SmallInt;
    nn:SmallInt;
  End; {record}

  TYUVreal=Packed record
    RY,RU,RV:Double;
  End; {record}

Function ConvertYUV(i:TYUVint):TYUVbyte; Overload;
Function ConvertYUV(i:TYUVbyte):TYUVint; Overload;
Function ConvertYUV(i:TYUVintn):TYUVint; Overload;
Function getYUVfromRGB(Red,Green,Blue:Integer):TYUVbyte; Overload;
Function getYUVfromRGB(p:RGBQUAD):TYUVbyte; Overload;
Procedure getRGBfromYUV(i:TYUVint; Var r,g,b:Integer); Overload;
Function getRGBfromYUV(i:TYUVbyte):RGBQUAD; Overload;
Procedure addYUV(Var r:TYUVreal; i:TYUVbyte);
Function compareYUV(i1,i2:TYUVintn):Boolean;
Function diffYUV(i1:TYUVbyte; i2:TYUVint):TYUVint;
Function deltaYUV(i1:TYUVbyte; i2:TYUVint):TYUVint; Overload;
Function deltaYUV(i1,i2:TYUVint):TYUVint; Overload;
Function squareYUV(i:TYUVint):Integer;
Procedure addmulYUV(Var i:TYUVbyte; factor:Extended; qe:TYUVint);
Function getcolorfromYUV(i:TYUVint):TColor;
Function getYUVfromColor(c:TColor):TYUVint;
Function IntToByte(i:Integer):Integer;

Implementation

Function IntToByte(i:Integer):Integer;
//ensures that the result is i being between the limits of a byte value
Begin
  Result:=i;
  if Result<0 then Result:=0;
  if Result>255 then Result:=255;
End;

Function RealToByte(r:Extended):Integer;
//ensures that the result is r being between the limits of a Byte value
Begin
  Result:=Round(r);
  if Result<0 then Result:=0;
  if Result>255 then Result:=255;
End;

//************* Converting routines between different YUV types ***********

Function ConvertYUV(i:TYUVint):TYUVbyte;
Begin
  Result.IY:=IntToByte(i.IY);
  Result.IU:=IntToByte(i.IU);
  Result.IV:=IntToByte(i.IV);
End;

Function ConvertYUV(i:TYUVbyte):TYUVint;
Begin
  Result.IY:=i.IY;
  Result.IU:=i.IU;
  Result.IV:=i.IV;
End;

Function ConvertYUV(i:TYUVintn):TYUVint;
Begin
  Result.IY:=i.IY;
  Result.IU:=i.IU;
  Result.IV:=i.IV;
End;

//************** Calculate YUV value of a Windows colour *******************

Function getYUVfromRGB(Red,Green,Blue:Integer):TYUVbyte;
Begin
  With Result Do Begin
    IY:=Round( 0.257*Red + 0.504*Green + 0.098*Blue)+16;
    IU:=Round(-0.148*Red - 0.291*Green + 0.439*Blue)+128;
    IV:=Round( 0.439*Red - 0.368*Green - 0.071*Blue)+128;
  End; {With}
End;

Function getYUVfromRGB(p:RGBQUAD):TYUVbyte;
//transforms a RGB value to a YUV triple
Begin
  With p Do
    Result:=getYUVfromRGB(rgbRed,rgbGreen,rgbBlue);
End;

Function getYUVfromColor(c:TColor):TYUVint;
Var
  Red,Green,Blue:Integer;
Begin
  Blue:= (c shr $10) And $FF;
  Green:=(c shr $08) And $FF;
  Red:=  c And $FF;
  Result:=ConvertYUV(getYUVfromRGB(Red,Green,Blue));
End;

//************** Calculate a Windows colour from a YUV triple **************

Procedure getRGBfromYUV(i:TYUVint; Var r,g,b:Integer);
//transforms a YUV triple back to RGB color values.
Var
  y,u,v:Integer;
  yz:Extended;
Begin
  With i Do Begin
    y:=IY-16;
    u:=IU-128;
    v:=IV-128;
    yz:=1.164*y;
    b:=RealToByte(yz+2.018*u);
    g:=RealToByte(yz-0.813*v-0.391*u);
    r:=RealToByte(yz+1.596*v);
  End; {With}
End;

Function getRGBfromYUV(i:TYUVbyte):RGBQUAD;
//transforms a YUV triple back to RGB color values.
Var
  y,u,v:Integer;
  yz:Extended;
Begin
  With i Do Begin
    y:=IY-16;
    u:=IU-128;
    v:=IV-128;
    yz:=1.164*y;
    Result.rgbBlue:=RealToByte(yz+2.018*u);
    Result.rgbGreen:=RealToByte(yz-0.813*v-0.391*u);
    Result.rgbRed:=RealToByte(yz+1.596*v);
    Result.rgbReserved:=0;
  End; {With}
End;

Procedure addYUV(Var r:TYUVreal; i:TYUVbyte);
//simply adds the YUV triple i to r.
Begin
  With r,i Do Begin
    RY:=RY+IY;
    RU:=RU+IU;
    RV:=RV+IV;
  End; {With}
End;

Function compareYUV(i1,i2:TYUVintn):Boolean;
//compares two YUV triples. TRUE Result if both are equal. The nn member is
//not compared.
Begin
  Result:=(i1.IY=i2.IY) And (i1.IU=i2.IU) And (i1.IV=i2.IV);
End;

Function diffYUV(i1:TYUVbyte; i2:TYUVint):TYUVint;
//results a YUV triple which is the signed difference between i1 And i2.
Begin
  Result.IY:=i1.IY-i2.IY;
  Result.IU:=i1.IU-i2.IU;
  Result.IV:=i1.IV-i2.IV;
End;

Function deltaYUV(i1:TYUVbyte; i2:TYUVint):TYUVint; Overload;
//results a YUV triple which is the delta between i1 And i2.
Begin
  Result.IY:=Abs(i1.IY-i2.IY);
  Result.IU:=Abs(i1.IU-i2.IU);
  Result.IV:=Abs(i1.IV-i2.IV);
End;

Function deltaYUV(i1,i2:TYUVint):TYUVint; Overload;
//results a YUV triple which is the delta between i1 And i2.
Begin
  Result.IY:=Abs(i1.IY-i2.IY);
  Result.IU:=Abs(i1.IU-i2.IU);
  Result.IV:=Abs(i1.IV-i2.IV);
End;

Function squareYUV(i:TYUVint):Integer;
//results the sum of the squared Y, the U And V values
Begin
  With i Do
    Result:=IY*IY+IU*IU+IV*IV;
End;

Procedure addmulYUV(Var i:TYUVbyte; factor:Extended; qe:TYUVint);
//adds the factor-multiplicated value of i2 to i1. i2 remains unchanged.
Begin
  With i Do Begin
    IY:=RealToByte(IY+factor*qe.IY);
    IU:=RealToByte(IU+factor*qe.IU);
    IV:=RealToByte(IV+factor*qe.IV);
  End; {With}
End;

Function getcolorfromYUV(i:TYUVint):TColor;
//results the Windows colour calculated from a YUV triple
Var
  r,g,b:Integer;
Begin
  getRGBfromYUV(i,r,g,b);
  Result:=RGB(r,g,b);
End;

End.
